#include <stdio.h>   	// Standard input/output definitions
#include <fcntl.h>   	// File control definitions
#include <termios.h> 	// POSIX terminal control definitions


/* Application who print the options of a TERMIOS device.

Example of application :
./Get_POSIX_Option /dev/ttyS0
./Get_POSIX_Option /dev/ttyUSB0
	
*/

int main(int argc, char *argv[])
{
	
	int fd; 							// File descriptor for the port
	struct termios options;						// Structure withe the option of the port
	speed_t baudRate;						// Baud rate of the device
	char *Speed = "unknown";					// String for printing the speed


      	fd = open(argv[1], O_RDWR | O_NOCTTY | O_NDELAY);		// Open port

      	if (fd == -1) {							// Test if the port is open
		perror("open_port: Unable to open device ");		// Return an error
		return fd;
	}

	fcntl(fd, F_SETFL, 0);
	printf ("Device successfully open\n\n");

	
	tcgetattr(fd, &options);					// Get port attributes



	/*** Baud rate ***/

     	baudRate = cfgetispeed(&options);				// Get the input baud rate
      
     	switch (baudRate) {
      		case B0:      Speed = "none"; break;
        	case B50:     Speed = "50 bauds"; break;
        	case B110:    Speed = "110 bauds"; break;
        	case B134:    Speed = "134 bauds"; break;
        	case B150:    Speed = "150 bauds"; break;
        	case B200:    Speed = "200 bauds"; break;
        	case B300:    Speed = "300 bauds"; break;
        	case B600:    Speed = "600 bauds"; break;
        	case B1200:   Speed = "1200 bauds"; break;
        	case B1800:   Speed = "1800 bauds"; break;
        	case B2400:   Speed = "2400 bauds"; break;
        	case B4800:   Speed = "4800 bauds"; break;
        	case B9600:   Speed = "9600 bauds"; break;
        	case B19200:  Speed = "19200 bauds"; break;
        	case B38400:  Speed = "38400 bauds"; break;
        	case B57600:  Speed = "57600 bauds"; break;
		case B115200: Speed = "115200 bauds"; break;
	}
	printf("Input speed = %s\n", Speed);

     	baudRate = cfgetospeed(&options);				// Get the output baud rate
      
     	switch (baudRate) {
      		case B0:      Speed = "none"; break;
        	case B50:     Speed = "50 bauds"; break;
        	case B110:    Speed = "110 bauds"; break;
        	case B134:    Speed = "134 bauds"; break;
        	case B150:    Speed = "150 bauds"; break;
        	case B200:    Speed = "200 bauds"; break;
        	case B300:    Speed = "300 bauds"; break;
        	case B600:    Speed = "600 bauds"; break;
        	case B1200:   Speed = "1200 bauds"; break;
        	case B1800:   Speed = "1800 bauds"; break;
        	case B2400:   Speed = "2400 bauds"; break;
        	case B4800:   Speed = "4800 bauds"; break;
        	case B9600:   Speed = "9600 bauds"; break;
        	case B19200:  Speed = "19200 bauds"; break;
        	case B38400:  Speed = "38400 bauds"; break;
        	case B57600:  Speed = "57600 bauds"; break;
		case B115200: Speed = "115200 bauds"; break;
	}
	printf("Output speed = %s\n", Speed);



	printf ("\n::: Input mode (c_iflag) :::\n");

	printf ("IGNBRK  %4x : %x\n",	IGNBRK,	(options.c_iflag & IGNBRK));
	printf ("BRKINT  %4x : %x\n",	BRKINT,	(options.c_iflag & BRKINT));
	printf ("IGNPAR  %4x : %x\n",	IGNPAR,	(options.c_iflag & IGNPAR));
	printf ("PARMRK  %4x : %x\n",	PARMRK,	(options.c_iflag & PARMRK));
	printf ("INPCK   %4x : %x\n",	INPCK,	(options.c_iflag & INPCK));
	printf ("ISTRIP  %4x : %x\n",	ISTRIP,	(options.c_iflag & ISTRIP));
	printf ("INLCR   %4x : %x\n",	INLCR,	(options.c_iflag & INLCR));
	printf ("IGNCR   %4x : %x\n",	IGNCR,	(options.c_iflag & IGNCR));
	printf ("ICRNL   %4x : %x\n",	ICRNL,	(options.c_iflag & ICRNL));
	printf ("IUCLC   %4x : %x\n",	IUCLC,	(options.c_iflag & IUCLC));
	printf ("IXON    %4x : %x\n",	IXON,	(options.c_iflag & IXON));
	printf ("IXANY   %4x : %x\n",	IXANY,	(options.c_iflag & IXANY));
	printf ("IXOFF   %4x : %x\n",	IXOFF,	(options.c_iflag & IXOFF));



	printf ("\n::: Output modes (c_oflag) :::\n"); 

	printf ("OPOST   %4x : %x\n",	OPOST,	(options.c_oflag & OPOST));
	printf ("OLCUC   %4x : %x\n",	OLCUC,	(options.c_oflag & OLCUC));
	printf ("ONLCR   %4x : %x\n",	ONLCR,	(options.c_oflag & ONLCR));
	printf ("OCRNL   %4x : %x\n",	OCRNL,	(options.c_oflag & OCRNL));
	printf ("ONOCR   %4x : %x\n",	ONOCR,	(options.c_oflag & ONOCR));
	printf ("ONLRET  %4x : %x\n",	ONLRET,	(options.c_oflag & ONLRET));
	printf ("OFILL   %4x : %x\n",	OFILL,	(options.c_oflag & OFILL));
	printf ("NLDLY   %4x : %x\n",	NLDLY,	(options.c_oflag & NLDLY));
	printf ("CRDLY   %4x : %x\n",	CRDLY,	(options.c_oflag & CRDLY));
	printf ("TABDLY  %4x : %x\n",	TABDLY,	(options.c_oflag & TABDLY));
	printf ("BSDLY   %4x : %x\n",	BSDLY,	(options.c_oflag & BSDLY));
	printf ("VTDLY   %4x : %x\n",	VTDLY,	(options.c_oflag & VTDLY));
	printf ("FFDLY   %4x : %x\n",	FFDLY,	(options.c_oflag & FFDLY));



	printf ("\n::: Control modes (c_cflag) :::\n");	

	printf ("CSIZE   %4x : %x\n",	CSIZE,	options.c_cflag & CSIZE);
	printf ("CSTOPB  %4x : %x\n",	CSTOPB,	options.c_cflag & CSTOPB);
	printf ("CREAD   %4x : %x\n",	CREAD,	options.c_cflag & CREAD);
	printf ("PARENB  %4x : %x\n",	PARENB,	options.c_cflag & PARENB);	
	printf ("PARODD  %4x : %x\n",	PARODD,	options.c_cflag & PARODD);	
	printf ("HUPCL   %4x : %x\n",	HUPCL,	options.c_cflag & HUPCL);	
	printf ("CLOCAL  %4x : %x\n",	CLOCAL,	options.c_cflag & CLOCAL);	



	printf ("\n::: Local modes (c_lflag) :::\n");

	printf ("ISIG   %4x : %x\n",	ISIG,	options.c_lflag & ISIG);
	printf ("ICANON %4x : %x\n",	ICANON,	options.c_lflag & ICANON);
	printf ("XCASE  %4x : %x\n",	XCASE,	options.c_lflag & XCASE);
	printf ("ECHO   %4x : %x\n",	ECHO,	options.c_lflag & ECHO);
	printf ("ECHOE  %4x : %x\n",	ECHOE,	options.c_lflag & ECHOE);
	printf ("ECHOK  %4x : %x\n",	ECHOK,	options.c_lflag & ECHOK);
	printf ("ECHONL %4x : %x\n",	ECHONL,	options.c_lflag & ECHONL);
	printf ("NOFLSH %4x : %x\n",	NOFLSH,	options.c_lflag & NOFLSH);
	printf ("TOSTOP %4x : %x\n",	TOSTOP,	options.c_lflag & TOSTOP);
	printf ("IEXTEN %4x : %x\n",	IEXTEN,	options.c_lflag & IEXTEN);



	printf ("\n::: Control chars (c_cc[NCCS]) :::\n");

	printf ("VEOF       : %d\n",		options.c_cc[VEOF]);
	printf ("VEOL       : %d\n",		options.c_cc[VEOL]);
	printf ("VERASE     : %d\n",		options.c_cc[VERASE]);
	printf ("VINTR      : %d\n",		options.c_cc[VINTR]);
	printf ("VKILL      : %d\n",		options.c_cc[VKILL]);
	printf ("VMIN       : %d\n",		options.c_cc[VMIN]);
	printf ("VQUIT      : %d\n",		options.c_cc[VQUIT]);
	printf ("VSTART     : %d\n",		options.c_cc[VSTART]);
	printf ("VSTOP      : %d\n",		options.c_cc[VSTOP]);
	printf ("VSUSP      : %d\n",		options.c_cc[VSUSP]);
	printf ("VTIME      : %d\n",		options.c_cc[VTIME]);

	close (fd);
      	return fd;
 }
